/*
 * This file is part of the YASMIN 8051 simulator project
 *
 * Copyright (c) 2000 by Gwenn Englebienne
 *
 * This software is free software; it is protected by the GPL. You can 
 * modify and/or redistribute it under the terms specified by the GNU
 * Public Licence version 2.
 *
 * Be advised that this software comes with no warranty at all, not
 * even the implied warranty of merchantability or fitness for a 
 * particular purpose. 
 *
 * For more information, you could look at the COPYING file, or have a 
 * look at the free software foundation's web site: http://www.fsf.org
 *
 */
 
#ifndef DEBUGWIDGET_H
#define DEBUGWIDGET_H

#include <qwidget.h>
#include <qpaintdevice.h>
#include <qfontmetrics.h>
#include <qfont.h>
#include <qtimer.h>
#include "linetext.h"
#include "hashtable.h"

class DebugWidget : public QWidget
{
     
    Q_OBJECT

 public:
     DebugWidget (QWidget *parent = 0, const char *name = 0,
		  const char *fileName = 0);
     ~DebugWidget ();

     void gotoNextLine ();
     void setCurLine (unsigned int nr);
     
     unsigned int getCurLineNum (void) { return currentLine; };
     char *getCurLine (void) { return txt.getChars (currentLine); };
     unsigned int getNumLines () { return txt.getNumLines(); };
     unsigned int getHeightInLines () {
          unsigned int tmp = height() / lh;
          if (tmp < txt.getNumLines ())
               return tmp;
          else
               return 1;
     };
     int curLineChanged ();
     
     int hasChanged () {
          int tmp = changed;
          changed = 0;
          return tmp;
     }
     
     int isBreak (unsigned int line_nr) { return txt.isBreakSet(line_nr); };
     unsigned char addBreak (unsigned int line_nr);
     unsigned char removeBreak (unsigned int line_nr);
     void setFileName (const char *fileName);
     char *getFileName() { return filename; };
     void loadFile (const char *filename);
     void saveFile (const char *filename);

     char *getLine (unsigned int nr) { return txt.getChars (nr); };
     void createLT (hashTable <unsigned int> *ht);

     virtual QSize sizeHint () const;
     virtual QSizePolicy sizePolicy() const;
     
 public slots:
     void blinkCursor();
     void loadFile ();
     void saveFile ();
     void newFile ();
     void changeOrigin (int y);

 signals:
     void originChange (int y);
     
 protected:

     LineText txt;
     QPoint origin, cursor, selStart;
     QFont *defFont;
     QFontMetrics *metrics;
     QTimer *blinker;
     QPainter *cpaint, *paint;
     QPixmap *pix;
     QImage *currentPix, *breakPix;
     int cursorOn, lh, selecting;
     unsigned int breaks[256];
     unsigned char numBreaks;
     int showln, showbp, showcl, clchanged, changed, draggin;
     unsigned int currentLine, indent, dragginLN;
     char str[51];
     char *filename;

     // Widget specific data structures and methods

     void eraseCursor();
     void draw();
     void drawLine();
     void drawLine (unsigned int i);
     
     virtual void paintEvent (QPaintEvent *e);
     virtual void keyPressEvent (QKeyEvent *e);
     virtual void resizeEvent (QResizeEvent *e);
     virtual void mousePressEvent (QMouseEvent *e);
     virtual void mouseMoveEvent (QMouseEvent *e);
     virtual void mouseReleaseEvent (QMouseEvent *e);
     virtual void focusInEvent (QFocusEvent *e);
     virtual void focusOutEvent (QFocusEvent *e);
     virtual bool focusNextPrevChild (bool next);
};     
	  
#endif

