/*
 * This file is part of the YASMIN 8051 simulator project
 *
 * Copyright (c) 2000 by Gwenn Englebienne
 *
 * This software is free software; it is protected by the GPL. You can 
 * modify and/or redistribute it under the terms specified by the GNU
 * Public Licence version 2.
 *
 * Be advised that this software comes with no warranty at all, not
 * even the implied warranty of merchantability or fitness for a 
 * particular purpose. 
 *
 * For more information, you could look at the COPYING file, or have a 
 * look at the free software foundation's web site: http://www.fsf.org
 *
 */
 
#include "general.h"
#include <iostream.h>
#include <stdio.h>

#define CY 0x80
#define AC 0x40

char conversion_buffer[10];

void uppercase (char *s)
{
     char *tmp = s;
     while (*tmp) {
          if (*tmp >= 'a' && *tmp <= 'z')
               *tmp = *tmp - 'a' + 'A';
          tmp++;
     }
}

int is_a_nr (const char* s)
{
     int base, last_is_indic = 1;
     int len = strlen(s);
     switch (s[len-1]) {
     case 'D':
     case 'd':
          base = 10;
          break;
     case 'H':
     case 'h':
          base = 16;
          break;
     case 'O':
     case 'o':
          base = 8;
          break;
     case 'B':
     case 'b':
          base = 2;
          break;
     default:
          base = 10;
          last_is_indic = 0;
          break;
     }
     len -= last_is_indic;
     const char *tmp = s;
     char result = 1, char_val;
     if (len < 1)
          result = 0;
     if (*tmp == '-') 
          tmp++, len--;
     for (int i = 0; i < len; i++) {
          if (*tmp >= 'A')
               char_val = *tmp - 'A' + 10;
          else
               char_val = *tmp - '0';
          if (char_val < 0 || char_val >= base)
               result = 0;
          tmp++;
     }
     return result;
}

char *hex_byte (const char c)
{
     snprintf (conversion_buffer, 10, "%X (%d)", (unsigned char)c, c);
     return conversion_buffer;
}

char *hex_word (const unsigned short s)
{
     snprintf (conversion_buffer, 10, "%4X", s);
     return conversion_buffer;
}

char *bin_byte (const char c)
{
     char *ptr = conversion_buffer;
     short mask = 0x0080;  // not a char because some systems take the carry bit!
     for (int i = 0; i < 8; i++, mask >>= 1)
          if (c & mask)
               *ptr++ = '1';
          else
               *ptr++ = '0';
     *ptr = 0;
     return conversion_buffer;
}

char *psw_readable (const unsigned char c)
{
     char *tmp = conversion_buffer;
     if ((c & CY) == CY)
          *tmp++ = 'C';
     else
          *tmp++ = ' ';
     if (c & AC)
          *tmp++ = 'A';
     else
          *tmp++ = ' ';
     if (c & 0x20)
          *tmp++ = 'F';
     else
          *tmp++ = ' ';
     *tmp++ = ' ';
     if (c & 0x10)
          *tmp++ = '1';
     else
          *tmp++ = '0';
     if (c & 0x08)
          *tmp++ = '1';
     else
          *tmp++ = '0';
     *tmp++ = ' ';
     if (c & 0x04)
          *tmp++ = 'V';
     else
          *tmp++ = ' ';
     if (c & 0x01)
          *tmp++ = 'P';
     else
          *tmp++ = ' ';
     *tmp = 0;
     return conversion_buffer;
}

char asm_to_char (const char *s)
{
     return (char) asm_to_int (s);
}

int asm_to_int (const char *s)
{
//     uppercase(s);
     int base, last_is_indic = 1;
     int len = strlen(s);
     switch (s[len-1]) {
     case 'D':
     case 'd':
          base = 10;
          break;
     case 'H':
     case 'h':
          base = 16;
          break;
     case 'O':
     case 'o':
          base = 8;
          break;
     case 'B':
     case 'b':
          base = 2;
          break;
     default:
          base = 10;
          last_is_indic = 0;
          break;
     }
     len -= last_is_indic;
     const char *tmp = s;
     int result = 0, char_val;
     int negative = 0;
     if (*tmp == '-') 
          negative = 1, tmp++, len--;
     for (int i = 0; i < len; i++) {
          if (*tmp >= 'A')
               char_val = *tmp - 'A' + 10;
          else
               char_val = *tmp - '0';
          if (char_val < 0 || char_val >= base) {
               cout << "Wrong constant! " << s << endl;
               return 0;
          }
          else 
               result = result * base + char_val;
          tmp++;
     }
     if (negative )
          result = -result;
     return result;
}   

