/*
 * This file is part of the YASMIN 8051 simulator project
 *
 * Copyright (c) 2000 by Gwenn Englebienne
 *
 * This software is free software; it is protected by the GPL. You can 
 * modify and/or redistribute it under the terms specified by the GNU
 * Public Licence version 2.
 *
 * Be advised that this software comes with no warranty at all, not
 * even the implied warranty of merchantability or fitness for a 
 * particular purpose. 
 *
 * For more information, you could look at the COPYING file, or have a 
 * look at the free software foundation's web site: http://www.fsf.org
 *
 */
 
#ifndef HASHTABLE_H
#define HASHTABLE_H

#include <iostream.h>

template <class T>

class hashTable {
 protected:
     
     struct node {
          node *next;
          char *key;
          T value;
     };
     
     typedef node* list;
     
     unsigned int hashSize;
     node **table;
     unsigned int hash (const char *s) {
          unsigned int sum = 0;
          const char *tmp = s;
          while (*tmp)
               sum += *tmp++;
          return sum % hashSize;
     };
     
 public:
     hashTable (unsigned int size = 30);
     int add (const char *key, const T& value); // returns 1 if ok, 0 if key allready used
     T& get (const char *key);            // returns NULL if not found;
     int contains (const char *key);
     
     void show (void);
     
     T& operator[] (const char *key);
     
     void free (void);

     ~hashTable(void);
};

template <class T>

hashTable<T>::hashTable (unsigned int size)
{
  hashSize = size;
  table = new list[size];
  for (unsigned int i = 0; i < hashSize; i++)
    table[i] = 0;
}
/*
template <class T>

unsigned int hashTable<T>::hash(const char *s)
{
  unsigned int sum = 0;
  const char *tmp = s;
  while (*tmp)
       sum += *tmp++;
  return sum % hashSize;
}
*/
template <class T>

int hashTable<T>::add (const char *key, const T& value)
{
  unsigned int index = hash(key);
  node *tmp = table[index], *prev;
  prev = tmp;
  while (tmp) {
    if (!strcmp(key, tmp->key))
      return 0;         /* Sorry, key allready exists */
    prev = tmp;
    tmp = tmp->next;
  }
  if (prev == tmp) {
    table[index] = new node;
    table[index]->key = new char[strlen(key)+1];
    strcpy(table[index]->key, key);
    table[index]->value = value;
    table[index]->next = 0;
  }
  else {
    prev->next = new node;
    tmp = prev->next;
    tmp->next = 0;
    tmp->key = new char[strlen(key)+1];
    strcpy(tmp->key, key);
    tmp->value = value;
  }
  return 1;
}

template <class T>

T& hashTable<T>::get(const char *key)
{
  unsigned int index = hash(key);
  node *tmp = table[index];
  while (tmp && strcmp(tmp->key, key))
    tmp = tmp->next;
  if (tmp)
    return tmp->value;
  else
    throw 1;
}

template <class T>

int hashTable<T>::contains(const char *key)
{
  node *tmp = table[hash(key)];
  while (tmp) {
    if (!strcmp(tmp->key, key))
      return 1;
    tmp = tmp->next;
  }
  return 0;
}

template <class T>

T& hashTable<T>::operator [] (const char *key)
{
     unsigned int index = hash(key);
     node *tmp = table[index];
     while (tmp && strcmp(tmp->key, key))
          tmp = tmp->next;
     if (tmp)
          return tmp->value;
     else if (tmp == table[index]) {
          table[index] = new node;
          tmp = table[index];
          tmp->next = 0;
          tmp->key = new char[strlen(key) + 1];
          strcpy(tmp->key, key);
          return tmp->value;
     } else {
          tmp = table[index];
          while (tmp->next) tmp = tmp->next;
          tmp->next = new node;
          tmp = tmp->next;
          tmp->key = new char [strlen(key) + 1];
          strcpy(tmp->key, key);
          tmp->next = 0;
          return tmp->value;
     }
}

template <class T>
void hashTable<T>::show(void)
{
  node *tmp;
  for (int i = 0; i < hashSize; i++) {
    tmp = table[i];
    cout << i << ": ";
    while (tmp) {
      cout << tmp->key << "<->" << (int)tmp->value << "  ";
      tmp = tmp->next;
    }
    cout << endl;
  }
}

template <class T>

void hashTable<T>::free (void)
{
     node *tmp, *prev;
     for (unsigned int i = 0; i < hashSize; i++) {
          tmp = table[i];
          while (tmp) {
               prev = tmp;
               tmp = tmp->next;
               delete prev;
          }
     }
     delete [] table;
     table = new list[hashSize];
     for (unsigned int i = 0; i < hashSize; i++)
          table[i] = 0;
}

template <class T>

hashTable<T>::~hashTable (void )
{
     node *tmp, *prev;
     for (unsigned int i = 0; i < hashSize; i++) {
          tmp = table[i];
          while (tmp) {
               prev = tmp;
               tmp = tmp->next;
               delete prev;
          }
     }
     delete [] table;
}

#endif

