/*
 * This file is part of the YASMIN 8051 simulator project
 *
 * Copyright (c) 2000 by Gwenn Englebienne
 *
 * This software is free software; it is protected by the GPL. You can 
 * modify and/or redistribute it under the terms specified by the GNU
 * Public Licence version 2.
 *
 * Be advised that this software comes with no warranty at all, not
 * even the implied warranty of merchantability or fitness for a 
 * particular purpose. 
 *
 * For more information, you could look at the COPYING file, or have a 
 * look at the free software foundation's web site: http://www.fsf.org
 *
 */
 
/*
  This source code is protected by the GNU Public Licence.
  See the file 'COPYING' for more details.

  Copyright 2000 by Gwenn Englebienne

  Enjoy.

  This is the code that shows the data memory of our simulated 8051
  in hexadecimal form.
*/

#include "hexview.h"
#include "memedit.h"
#include <stdio.h>

#define MAXORIGIN 32
#define INTERSIGN 5
#define INDENT 50

HexView::HexView (accessType t, QWidget *parent, const char *name)
     : QWidget (parent, name)
{
     at = t;
     setMinimumSize (200, 20);
     paint = new QPainter (this);
     fnt = new QFont ("lucidatypewriter");
     metrics = new QFontMetrics (*fnt);
     paint->setFont (*fnt);
     paint->setBackgroundColor (QColor (255, 255, 255));
     lh = metrics->height();
     origin = 0;
}

HexView::~HexView ()
{
     delete metrics;
     delete fnt;
     delete paint;
}

void HexView::originChange (int val)
{
     if (val < MAXORIGIN)
          origin = val;
     paintEvent (0);
}


void HexView::resizeEvent (QResizeEvent *e)
{
     int max = 32 - (height() / lh);
     if (max < 0)
          max = 0;
     emit newMax (max);
}

void HexView::paintEvent (QPaintEvent *e)
{
     char buf[5];
     int signWidth = metrics->width ("AA") + INTERSIGN;
     int i;
//     paint->eraseRect (0, 0, width(), height());
     paint->setClipRect(2,2,width()-4, height()-4);
     paint->fillRect (0, 0, INDENT-5, height(), lightGray);
     for (i = origin * 8; i < 8 * (origin + 1 +(height() / lh )) && i < 256
               ; i++) {
          if (at == direct)
               snprintf (buf, 3, "%X", (unsigned char)da(i));
          else
               snprintf (buf, 3, "%X", (unsigned char)ida(i));               
          paint->eraseRect (INDENT + (i % 8) * signWidth - 5,
                            ((i / 8) - origin) * lh, signWidth, lh);
          paint->drawText (INDENT + (i % 8) * signWidth,
                           ((i / 8) - origin + 1) * lh, buf);
     }
     for (i = 0; (i + origin) < 32 && i < height () / lh + 1; i++) {
          snprintf (buf, 5, "0x%X", ((i + origin) * 8));
//          paint->eraseRect (0, i * lh, INDENT - 6, lh);
          paint->drawText (4, (i + 1) * lh, buf);
     }
     paint->setClipRect (0, 0, width(), height());
     paint->eraseRect (INDENT, i * lh, width()-INDENT, lh);
     paint->setPen (darkBlue);
     paint->drawLine (INDENT - 5, 0, INDENT - 5, height());
     paint->setPen (QPen (black, 2));
     paint->drawLine (0, 0, width(), 0);
     paint->drawLine (0, 0, 0, height());
     paint->setPen (QPen (white, 2));
     paint->drawLine (0, height(), width (), height ());
     paint->drawLine (width (), 0, width (), height ());
     paint->setPen (black);
}          

void HexView::update ()
{
     paintEvent (0);
}

QSizePolicy HexView::sizePolicy () const
{
     return QSizePolicy (QSizePolicy::Fixed, QSizePolicy::Expanding);
}

QSize HexView::sizeHint () const
{
     return QSize (200, 200);
}

void HexView::mouseDoubleClickEvent (QMouseEvent *e)
{
     if (e->x() > INDENT) {
          int i = (e->x() - INDENT) / (metrics->width("AA") + INTERSIGN) +
               ((e->y() / lh) + origin) * 8;
          char buf[15];
          snprintf (buf, 15, "Change Ox%X", i);
          MemEdit *me;
          if (at == direct)
               me = new MemEdit (i, buf, MemEdit::direct, this);
          else
               me = new MemEdit (i, buf, MemEdit::indirect, this);
          if (me->exec())
               update ();
     }
}



