/*
 * This file is part of the YASMIN 8051 simulator project
 *
 * Copyright (c) 2000 by Gwenn Englebienne
 *
 * This software is free software; it is protected by the GPL. You can 
 * modify and/or redistribute it under the terms specified by the GNU
 * Public Licence version 2.
 *
 * Be advised that this software comes with no warranty at all, not
 * even the implied warranty of merchantability or fitness for a 
 * particular purpose. 
 *
 * For more information, you could look at the COPYING file, or have a 
 * look at the free software foundation's web site: http://www.fsf.org
 *
 */
 
#include "int_stack.h"
#include <iostream.h>

// #define NULL 0

isr_stack::isr_stack(void)
{
     root = NULL;
}

isr_stack::~isr_stack (void)
{
     stack_node *tmp;
     if (root) {
	  while (root->next) {
	       tmp = root->next;
	       root->next = tmp->next;
	       delete tmp;
	  }
	  delete root;
     }
}
	  

void isr_stack::free (void)
{
     stack_node *tmp;
     if (root) {
	  while (root->next) {
	       tmp = root->next;
	       root->next = tmp->next;
	       delete tmp;
	  }
	  delete root;
     }
     root = NULL;
}

void isr_stack::push (isr t, int p)
{
     stack_node *tmp;
     if (root) {
	  tmp = root;
	  while (tmp->next)
	       tmp = tmp->next;
	  tmp->next = new stack_node;
	  tmp = tmp->next;
	  tmp->type = t;
	  tmp->HP = p;
	  tmp->next = NULL;
     }
     else {
	  root = new stack_node;
	  root->type = t;
	  root->HP = p;
	  root->next = NULL;
     }
}

isr isr_stack::pop (void)
{
     isr rv;
     if (root && root->next) {
	  stack_node *tmp = root;
	  while (tmp->next->next)
	       tmp = tmp->next;
	  rv = tmp->next->type;
	  delete tmp->next;
	  tmp->next = NULL;
     }
     else {
	  rv = root->type;
	  delete root;
	  root = NULL;
     }
     return rv;
}

int isr_stack::contains (isr t)
{
     stack_node *tmp = root;
     while (tmp) {
	  if (tmp->type == t)
	       return 1;
	  tmp = tmp->next;
     }
     return 0;
}
	  
int isr_stack::contains_higher (isr t, int p)
{
     stack_node *tmp = root;
     while (tmp) {
          if (tmp->HP > p || (tmp->HP == p && tmp->type >= t))
               return 1;
          tmp = tmp->next;
     }
     return 0;
}

void isr_stack::show(void)
{
     stack_node *tmp = root;
     cout << " ||";
     while (tmp) {
	  cout << "-> val = " << tmp->type << ", P = " << tmp->HP;
	  tmp = tmp->next;
     }
     cout << endl;
}
