/*
 * This file is part of the YASMIN 8051 simulator project
 *
 * Copyright (c) 2000 by Gwenn Englebienne
 *
 * This software is free software; it is protected by the GPL. You can 
 * modify and/or redistribute it under the terms specified by the GNU
 * Public Licence version 2.
 *
 * Be advised that this software comes with no warranty at all, not
 * even the implied warranty of merchantability or fitness for a 
 * particular purpose. 
 *
 * For more information, you could look at the COPYING file, or have a 
 * look at the free software foundation's web site: http://www.fsf.org
 *
 */
 
#include "ledbar.h"

#define border 5

Ledbar::Ledbar (QWidget *parent, const char *name)
     : QWidget (parent, name)
{
     setMaximumSize (600, 20);
     setMinimumSize (50, 20);
     val = 0;
}

Ledbar::Ledbar (unsigned char val, QWidget *parent, const char *name)
     : QWidget (parent, name)
{
     this->val = val;
}

void Ledbar::setVal (unsigned char val)
{
     if (this->val != val) {
          this->val = val;
          emit valueChanged(val);
          paintEvent (0);
     }
}

void Ledbar::paintEvent (QPaintEvent *e)
{
     QPainter paint(this);
     int i, w = (width() - border) / 8;
     paint.setPen (Qt::black);
     for (i = 0; i < 8; i++)
          if (val & (1 << i)) {
               paint.fillRect (border + ((7-i) * w), border, w - border,
                               height() - 2 * border, QColor (200, 200, 255));
//               paint.setPen (QColor (230, 100, 100));
               paint.drawRect (border + ((7-i) * w), border, w - border,
                               height() - 2 * border);
//               paint.setPen (Qt::black);

          }
          else {
               paint.fillRect (border + ((7-i) * w), border,
                               w - border, height() - 2 *border, Qt::darkGray);
               paint.drawRect (border + ((7-i) * w), border, w - border,
                               height() - 2 * border);
          }
}

void Ledbar::mousePressEvent (QMouseEvent *e)
{
     int i;
     if (e->y() > border && e->y() < height() - border) {
          i = 7 - (e->x() - border) * 8 / (width() - 2*border);
          if (val & (1 << i))
               val &= ~(1 << i);
          else
               val |= (1 << i);
          emit valueChanged ((unsigned char)val);
          paintEvent (0);
     }
     
}







