/*
 * This file is part of the YASMIN 8051 simulator project
 *
 * Copyright (c) 2000 by Gwenn Englebienne
 *
 * This software is free software; it is protected by the GPL. You can 
 * modify and/or redistribute it under the terms specified by the GNU
 * Public Licence version 2.
 *
 * Be advised that this software comes with no warranty at all, not
 * even the implied warranty of merchantability or fitness for a 
 * particular purpose. 
 *
 * For more information, you could look at the COPYING file, or have a 
 * look at the free software foundation's web site: http://www.fsf.org
 *
 */
 
#include "linetext.h"

#include <iostream.h>
#include <stdio.h>

// char buf[50];

LineText::LineText()
{
     nol = 0;
     for (int i = 0; i < CUTS; i++)
          lines[i] = 0;
}

LineText::~LineText()
{
     line *tmp = lines[0], *nxt;
     while (tmp) {
          nxt = tmp->down;
          delete tmp->data;
          delete tmp;
          tmp = nxt;
     }
}

line *LineText::getLine (unsigned int nr)
{

     if (nr >= nol) {
	  cerr << "Line nr (" << nr << ") >= number of lines (" << nol <<")\n";
	  throw new int (1);
     }

     unsigned char i = nr % CUTS;
     line *tmp = lines[i];
     for (int j = 0; j < nr / CUTS; j++) {
	  if (tmp == 0) {
//	       snprintf (buf, 50, "Unknown line nr (%u) in DebugWidget", nr);
	       throw new int (2);
	  }
	  tmp = tmp->right;
     }
     return tmp;
}

void LineText::deleteLine (unsigned int nr)
{
     line *l = getLine (nr);
     line *right = l->right;
     
     if (l->up)
          l->up->down = l->down;
     if (l->down)
          l->down->up = l->up;
     
     if (nr < CUTS) {
          for (int i = nr; i < CUTS - 1; i++)
               lines[i] = lines[i+1];
          lines[CUTS - 1] = lines[0]->right;
          if (lines[0]->right)
               lines[0]->right->left = 0;
          line *prev =lines[0];
          line *tmp = lines[CUTS-1];
          if (tmp)
               tmp = tmp->down;
          while (tmp && tmp != l->right) {
               tmp->left = prev;
               tmp->left->right = tmp;
               tmp = tmp->down;
               prev = prev->down;
          }
          if (tmp) { // tmp == l->right
               tmp->left = prev;
               tmp->left->right = tmp;
          }
          else {
               prev->right = 0;
          }
     }
     else {
          line *prev = l->left;
          line *tmp;
          
          for (tmp = l->down; tmp && tmp != l->right; tmp = tmp->down) {
               tmp->left = prev;
               tmp->left->right = tmp;
               prev = prev->down;
          }
          if (tmp) { // tmp == l->right
               tmp->left = prev;
               tmp->left->right = tmp;
          }
          else {
               prev->right = 0;
          }
     }
     nol--;
     delete l;
}

line * LineText::insertLine (unsigned int nr)
{
     line *ins, *l;
     if (nr == nol) { // add last line.
          ins = new line;
          ins->length = 0;
          ins->bufferlength = 0;
          ins->data = 0;
          ins->breakpoint = 0;
          try {
               l = getLine (nr - 1);
          } catch (int *i) {
               if (nr != 0)
                    throw;
               // ignore this for once..
               
               l = 0;
          }
          if (l) {  // not the first line of the text
               l->down = ins;
               ins->up = l;
               if (nr < CUTS) {
                    lines[nr] = ins;
                    ins->left = 0;
               }
               else {
                    line *tmp = lines[nr % CUTS];
                    while (tmp->right)
                         tmp = tmp->right;
                    tmp->right = ins;
                    ins->left = tmp;
               }
          }
          else {   // the very first line.
               lines[0] = ins;
               ins->up = 0;
               ins->left = 0;
          }
          ins->down = 0;
          ins->right = 0;
          nol++;
          return ins;
     }
     
     try {
          l = getLine (nr);
     } catch (int *c) {
          cout << "Exception catched: " << c << endl;
          exit(1);
     }
     ins = new line;
     ins->data = 0;
     ins->length = 0;
     ins->bufferlength = 0;
     ins->breakpoint = 0;
     nol++;
     if (l->up)
	  l->up->down = ins;
     ins->up = l->up;
     l->up = ins;
     ins->down = l;
     if (nr < CUTS) {
          for (int i = CUTS - 1; i > nr; i--)
	       lines[i] = lines[i-1];
          ins->left = 0;
          ins->right = lines[nr]->right;
          lines[nr] = ins;
          line *l = lines[0];
          line *tmp = (lines[CUTS - 1] ? lines[CUTS-1]->down : 0);
          while (tmp && l != ins->down) {
               tmp->left = l;
               tmp->left->right = tmp;
               tmp = tmp->down;
               l = l->down;
          }
     }
     else {
          line *tmp = ins;
          tmp->right = 0;
          line *next = l->left;
          while (tmp && tmp != l->right) {
               tmp->left = next;
               tmp->left->right = tmp;
               tmp = tmp->down;
               next = next->down;
          }
          if (!tmp)
               next->right = 0;
     }
     return ins;
}

void LineText::setData (line *l, char *buf)
{
     if (l->data)
	  delete l->data;
     l->data = new char[strlen(buf)+1];
     l->length = strlen(buf);
     l->bufferlength = l->length;
     strcpy(l->data, buf);
}

void LineText::print()
{
     cout << "Number of lines = " << nol << endl;
     line *r;

     for (int i = 0; i < CUTS; i++) {
	  r = lines[i];
	  while (r) {
	       cout << r->data << " (" << r->length << ") ";
	       if (r->right && r->right->left == r)
		    cout << " <--> ";
	       else
		    cout << " --> ";
	       r = r->right;
	  }
	  cout << "(earth)\n";
     }

/*     r = lines[0];
     while (r) {
	  if (r->left)
	       cout << r->left->data << (r->left->right == r ? " <--> " : " <-- ");
	  else
	       cout << "E <-- ";
	  cout << r->data;
	  if (r->right)
	       cout << (r->right->left == r ? " <--> " : " --> ") << r->right->data;
	  else
	       cout << " --> (E)";
	  cout << endl;
	  r = r->down;
	  }*/
}
    
void LineText::insertChar (char c, unsigned int line_nr, unsigned short pos)
{
     line *l;
     try {
	  l = getLine (line_nr);
     } catch (int *c) {
	  l = insertLine (line_nr);
     }
     insertChar (c, l, pos);
}

void LineText::insertChars (char *c, unsigned int line_nr, unsigned short pos)
{
     line *l;
     if (line_nr == nol) 
	  l = insertLine (line_nr);
     else
	  l = getLine (line_nr);
     insertChars (c, l, pos);
}       

void LineText::deleteChar (unsigned int line_nr, unsigned short pos)
{
     deleteChar (getLine (line_nr), pos);
}

void LineText::insertChar (char c, line *l, unsigned short pos)
{
     if (!l) {
//	  snprintf(buf, 50, "insertChar (...) called with line null!");
	  throw new int(4);
     }

     int s;
     
     if (l->bufferlength > pos + 1) {
	  for (s = l->length + 1; s > pos; s--)
	       l->data[s] = l->data[s-1];
	  l->data[pos] = c;
	  l->length++;
     }
     else {
	  l->bufferlength += INCREASE;
	  l->length++;
	  char *tmp = l->data;
	  l->data = new char [l->bufferlength];
	  if (tmp) {
	       for (s = l->length + 1; s > pos; s--)
		    l->data[s] = tmp[s-1];
	       l->data[s--] = c;
	       while (s >= 0) {
		    l->data[s] = tmp[s];
		    s--;
	       }
	  }
	  else {
	       l->data[0] = c;
	       l->data[1] = 0;
	  }
     }
}

void LineText::insertChars (char *c, line *l, unsigned short pos)
{
     if (!l) {
//	  snprintf(buf, 50, "insertChar (...) called with line null!");
          throw new int (5);
     }

     if (pos > l->length) {
          cerr << "Error [LineText::insertChar(char *c, line *l, unsigned short pos)]:\n               pos >  line length!!\n";
//	  snprintf(buf, 50, "insertChar(...): pos > line length!!");
          throw new int (6);
     }

     int s, cs;

     if (l->bufferlength > l->length + strlen (c)) {
          for (s = l->length + strlen(c); s >= (int)pos + (int)strlen(c); s--)
               l->data[s] = l->data[s-strlen(c)];
          cs = strlen (c) - 1;
          while (s >= pos) {
               l->data[s] = c[cs];
               s--; cs--;
          }
          l->length += strlen(c);
     }
     else {
          while (l->bufferlength < l->length + strlen(c) + 1) 
               l->bufferlength += INCREASE;
          l->length += strlen(c);
          char *tmp = l->data;
          l->data = new char [l->bufferlength];
          if (tmp) {
               for (s = l->length; s > (int)pos + (int)strlen(c) - 1; s--)
                    l->data[s] = tmp[s-strlen(c)];
               cs = strlen (c) - 1;
               while (s >= pos) {
                    l->data[s] = c[cs];
                    s--; cs--;
               }
               while (s >= 0) {
                    l->data[s] = tmp[s];
                    s--;
               }
               delete [] tmp;
          }
          else {
               strcpy (l->data, c);
          }
     }
}

void LineText::deleteChar (line *l, unsigned short pos)
{
     if (!l) {
//	  snprintf (buf, 50, "error in LineText::deleteLine (line *, unsigned short pos)\n\t\tline * == null\n");
	  throw new int(7);
     }

     unsigned short s;
     for (s = pos; s < l->length; s++)
	  l->data[s] = l->data[s+1];
     l->length--;
}

char *LineText::getChars (unsigned int line_nr)
{
     return getLine(line_nr)->data;
}

char *LineText::getLeadingBlanks (unsigned int line_nr)
{
     line *l = getLine (line_nr);
     char *tmp = l->data, *ret;
     int len = 0;
     
     if (tmp && (*tmp == ' ' || *tmp == '\t')) {
          while (*tmp && (*tmp == ' ' || *tmp == '\t'))
               tmp++, len++;
          ret = new char[len+1];
          char *dst = ret;
          tmp = l->data;
          while (*tmp && (*tmp == ' ' || *tmp == '\t')) {
               *dst = *tmp;
               tmp++; dst++;
          }
          *dst = 0;
     }
     else 
          ret = 0;  
     return ret;
}

void LineText::splitLine (unsigned int line_nr, unsigned short pos)
{
     line *first = getLine (line_nr);
     line *second = insertLine (line_nr + 1);
     if (first->data) {
	  if (pos > first->length) {
	       cerr << "Splitting after the end of the line\n";
	       throw new int (8);
	  }
	  setData (second, first->data + pos);
	  first->data[pos] = 0;
	  first->length = pos;
     }
}

void LineText::mergeLines (unsigned int line_nr)
{
     line *first = getLine (line_nr);
     if (!first) {
	  cerr << line_nr << " is a null line !?\n";
	  throw new int (9);
     }
	  
     line *second = first->down;
     if (!second) {
	  cerr << line_nr+1 << " does not exist!!!!\n";
	  throw new int (10);
     }

     insertChars (second->data, first, first->length);
     deleteLine (line_nr + 1);
}

void LineText::free(void)
{
     line *tmp = lines[0], *nxt;
     while (tmp) {
          nxt = tmp->down;
          if (tmp->data)
               delete [] tmp->data;
          delete tmp;
          tmp = nxt;
     }
     for (int i = 0; i < CUTS; i++)
          lines[i] = 0;
     nol = 0;
}
