/*
 * This file is part of the YASMIN 8051 simulator project
 *
 * Copyright (c) 2000 by Gwenn Englebienne
 *
 * This software is free software; it is protected by the GPL. You can 
 * modify and/or redistribute it under the terms specified by the GNU
 * Public Licence version 2.
 *
 * Be advised that this software comes with no warranty at all, not
 * even the implied warranty of merchantability or fitness for a 
 * particular purpose. 
 *
 * For more information, you could look at the COPYING file, or have a 
 * look at the free software foundation's web site: http://www.fsf.org
 *
 */
 
#ifndef LINETEXT_H
#define LINETEXT_H

#define CUTS           128
#define INCREASE       10

struct line {
     char *data;
     unsigned short length, bufferlength;
     line *up, *down, *left, *right; // up and down are obvious. Right is a shortcut to line + CUTS
     int breakpoint;
};

class LineText {
 protected:
     line *lines[CUTS];
     unsigned int nol; // # of lines

 public:
     LineText ();
     ~LineText();
     
     line *getLine (unsigned int nr);
     char *getChars (unsigned int line_nr);

     void deleteLine (unsigned int nr);
     line * insertLine (unsigned int nr);
     void setData (line *l, char *buf);
     
     void insertChar (char c, unsigned int line_nr, unsigned short pos);
     void insertChar (char c, line *l, unsigned short pos);
     void deleteChar (unsigned int line_nr, unsigned short pos);
     void deleteChar (line *l, unsigned short pos);
     char * getLeadingBlanks (unsigned int line_nr);
     
     void splitLine (unsigned int line_nr, unsigned short pos);
     void mergeLines (unsigned int line_nr);
     
     void insertChars (char *c, unsigned int line_nr, unsigned short pos);
     void insertChars (char *c, line *l, unsigned short pos);
     unsigned int getNumLines() { return nol; };
     unsigned short lineLength (unsigned int nr) {
          return getLine (nr)->length; };

     void setBreak (unsigned int line_nr) {
          getLine (line_nr)->breakpoint = 1;
     };
     void delBreak (unsigned int line_nr) {
          getLine (line_nr)->breakpoint = 0;
     };
     void reverseBreak (unsigned int line_nr) {
          line *l = getLine (line_nr);
          l->breakpoint = !l->breakpoint;
     }
     int isBreakSet (unsigned int line_nr) {
          return getLine(line_nr)->breakpoint;
     };

     void free(void);
     
     void print();
};

#endif
