/*
 * This file is part of the YASMIN 8051 simulator project
 *
 * Copyright (c) 2000 by Gwenn Englebienne
 *
 * This software is free software; it is protected by the GPL. You can 
 * modify and/or redistribute it under the terms specified by the GNU
 * Public Licence version 2.
 *
 * Be advised that this software comes with no warranty at all, not
 * even the implied warranty of merchantability or fitness for a 
 * particular purpose. 
 *
 * For more information, you could look at the COPYING file, or have a 
 * look at the free software foundation's web site: http://www.fsf.org
 *
 */
 
#include "mainwidget.h"
#include <qlabel.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include "xpms.inc"

MainWidget::MainWidget (QWidget *parent, const char *name)
     : QMainWindow (parent, name)
{
     mb = new QMenuBar (this);
     QLabel *junk = new QLabel("Doei", this);
     QLabel *j2 = new QLabel("Hello", this);
     sim = new Sim8051 (this);
     setCentralWidget (sim);

     file = new QPopupMenu (this);
     file->insertItem ("&New source file", sim, SLOT (newFile ()), CTRL+Key_N);
     file->insertItem ("&Open", sim, SLOT (load ()), CTRL+Key_O);
     file->insertItem ("&Save", sim, SLOT (save ()), CTRL+Key_S);
     file->insertItem ("Save &as...", sim, SLOT (saveAs ()), CTRL+Key_A);
     file->insertSeparator ();
     file->insertItem ("E&xit", this, SLOT (quit ()), CTRL+Key_X);
     mb->insertItem ("File", file);

     debug = new QPopupMenu (this); 
     debug->insertItem ("&Check syntax", sim, SLOT (check ()));
     mb->insertItem ("Debug", debug);
     QToolBar *tb = new QToolBar ("Tool Bar", this);
     QToolButton *nb = new QToolButton (QPixmap (newXpm), "New", "Grouptext1",
                                        sim, SLOT (newFile ()), tb);
     nb->setUsesBigPixmap (true);
     QToolButton *ob = new QToolButton (QPixmap (openXpm), "Open", "Grouptext1",
                                        sim, SLOT (load ()), tb);
     ob->setUsesBigPixmap (true);
     QToolButton *sb = new QToolButton (QPixmap (saveXpm), "Save", "Grouptext1",
                                        sim, SLOT (save ()), tb);
     sb->setUsesBigPixmap (true);
     tb->addSeparator ();
     QToolButton *stepb = new QToolButton (QPixmap (stepXpm), "Step Into", "Grouptext",
                                           sim, SLOT (stepIn ()), tb);
     stepb->setUsesBigPixmap (true);
     QToolButton *stepOutb = new QToolButton (QPixmap (stepOutXpm), "Step Out", "Grouptext",
                                           sim, SLOT (stepOut ()), tb);
     stepOutb->setUsesBigPixmap (true);
     QToolButton *stepOverb = new QToolButton (QPixmap (stepOverXpm), "Step Over", "Grouptext",
                                           sim, SLOT (stepOver ()), tb);
     stepOverb->setUsesBigPixmap (true);

     QToolButton *rb = new QToolButton (QPixmap (runXpm), "Run", "Grouptext",
                                        sim, SLOT (run ()), tb);
     rb->setUsesBigPixmap (true);
     QToolButton *rfb = new QToolButton (QPixmap (runfastXpm), "Run without output",
                                         "Grouptext", sim, SLOT (runFast ()), tb);
     rfb->setUsesBigPixmap (true);
     QToolButton *stopb = new QToolButton (QPixmap (stopXpm), "Stop running",
                                           "Grouptext",
                                           sim, SLOT (stop ()), tb);
     stopb->setUsesBigPixmap (true);
     QToolButton *resetb = new QToolButton (QPixmap (resetXpm), "Reset", "Grouptext",
                                           sim, SLOT (reset ()), tb);
     resetb->setUsesBigPixmap (true);

     QToolButton *quitb = new QToolButton (QPixmap (quitXpm), "Exit", "Grouptext",
                                           this, SLOT (quit ()), tb);
     quitb->setUsesBigPixmap (true);
     tb->addSeparator();

}

MainWidget::~MainWidget ()
{
     delete mb;
     delete sim;
}


void MainWidget::quit (void)
{
     exit (0);
}

