/*
 * This file is part of the YASMIN 8051 simulator project
 *
 * Copyright (c) 2000 by Gwenn Englebienne
 *
 * This software is free software; it is protected by the GPL. You can 
 * modify and/or redistribute it under the terms specified by the GNU
 * Public Licence version 2.
 *
 * Be advised that this software comes with no warranty at all, not
 * even the implied warranty of merchantability or fitness for a 
 * particular purpose. 
 *
 * For more information, you could look at the COPYING file, or have a 
 * look at the free software foundation's web site: http://www.fsf.org
 *
 */
 
#include "memhex.h"

MemHex::MemHex (QWidget *parent, const char *name)
     : QGrid (2, parent, name)
{
     setSpacing (5);
     direct = new QLabel ("Direct access memory:",this);
     j1 = new QLabel ("", this);
     viewer1 = new HexView (HexView::direct, this);
     sb1 = new QScrollBar (0, 32, 1, 10, 0, QScrollBar::Vertical, this);
     connect (sb1,SIGNAL (valueChanged(int)),viewer1,SLOT (originChange(int)));
     connect (viewer1, SIGNAL (newMax (int)), this, SLOT (newMax1 (int)));
     indirect = new QLabel ("Indirect access memory:", this);
     j2 = new QLabel ("", this);
     viewer2 = new HexView (HexView::indirect, this);
     sb2 = new QScrollBar (0, 32, 1, 10, 0, QScrollBar::Vertical, this);
     connect (sb2, SIGNAL (valueChanged (int)), viewer2,
              SLOT (originChange (int)));
     connect (viewer2, SIGNAL (newMax (int)), this, SLOT (newMax2 (int)));

}

MemHex::~MemHex ()
{
     delete direct;
     delete j1;
     delete viewer1;
     delete sb1;
     delete indirect;
     delete j2;
     delete viewer2;
     delete sb2;
}

void MemHex::newMax1 (int val)
{
     sb1->setRange (0, val);
}

void MemHex::update ()
{
     viewer1->update ();
     viewer2->update ();
}

void MemHex::newMax2 (int val)
{
     sb2->setRange (0, val);
}

