/*
 * This file is part of the YASMIN 8051 simulator project
 *
 * Copyright (c) 2000 by Gwenn Englebienne
 *
 * This software is free software; it is protected by the GPL. You can 
 * modify and/or redistribute it under the terms specified by the GNU
 * Public Licence version 2.
 *
 * Be advised that this software comes with no warranty at all, not
 * even the implied warranty of merchantability or fitness for a 
 * particular purpose. 
 *
 * For more information, you could look at the COPYING file, or have a 
 * look at the free software foundation's web site: http://www.fsf.org
 *
 */
 
#ifndef MEMORY_H
#define MEMORY_H

#include "hashtable.h"

#define   DEBUG  0
#define   A      0xE0
#define   B      0xF0
#define   PSW    0xD0
#define   SP     0x81
#define   DPL    0x82
#define   DPH    0x83
#define   P0     0x80
#define   P1     0x90
#define   P2     0xA0
#define   P3     0xB0
#define   IP     0xB8
#define   IE     0xA8
#define   TMOD   0x89
#define   TCON   0x88
#define   TH0    0x8C
#define   TL0    0x8A
#define   TH1    0x8D
#define   TL1    0x8B
#define   SCON   0x98
#define   SBUF   0x99
#define   PCON   0x87

enum sfr_type { bit, byte, word };

struct SFR {
  sfr_type type;
  unsigned char address;
};

extern hashTable <SFR> sfr_info;

char& da (unsigned char index);
char& ida (unsigned char index);

void set_bit (unsigned char address);

void clr_bit (unsigned char address);
int is_bit_set (unsigned char address);
void cpl_bit (unsigned char address);
unsigned short get_dptr (void);
void set_dptr (unsigned short val);

void create_sfr_with_bit_address (const char *name, unsigned char address);
void create_r_registers (void);
void create_sfr_table (void);

#endif // MEMORY_H

