/*
 * This file is part of the YASMIN 8051 simulator project
 *
 * Copyright (c) 2000 by Gwenn Englebienne
 *
 * This software is free software; it is protected by the GPL. You can 
 * modify and/or redistribute it under the terms specified by the GNU
 * Public Licence version 2.
 *
 * Be advised that this software comes with no warranty at all, not
 * even the implied warranty of merchantability or fitness for a 
 * particular purpose. 
 *
 * For more information, you could look at the COPYING file, or have a 
 * look at the free software foundation's web site: http://www.fsf.org
 *
 */
 
#include "memview.h"
#include "general.h"
#include "memory.h"
#include "memedit.h"
#include <stdio.h>
#include <iostream.h>

MemView::MemView (QWidget *parent, const char *name)
     : QGrid (1, parent, name)
{
     setMaximumSize (300, 1200);
     setMinimumSize (200, 300);
     setSpacing (5);

     title = new QLabel ("Registers:", this);
     lv = new QListView (this);
     lv->addColumn ("Register name");
     lv->addColumn ("Value");
     connect (lv, SIGNAL (rightButtonClicked (QListViewItem *,
                                              const QPoint &,
                                              int)),
              this, SLOT (singleClick (QListViewItem *,
                                       const QPoint &,
                                       int)));

     acc = new QListViewItem (lv, "Accumulator", "");
     accDec = new QListViewItem (acc, "decimal", "0");
//     accHex = new QListViewItem (acc, "hexadecimal", "0h");

     b = new QListViewItem (lv, "B register", "");
     bDec = new QListViewItem (b, "decimal", "0");
//     bHex = new QListViewItem (b, "hexadecimal", "0h");
    
     psw = new QListViewItem (lv, "Program status word", "");
     pswDec = new QListViewItem (psw, "decimal", "0");
     pswHex = new QListViewItem (psw, "hexadecimal", "0h");
//     pswReadable = new QListViewItem (psw, "bitwise meaning", "0h");
   
     sp = new QListViewItem (lv, "Stack Pointer", "");
     spDec = new QListViewItem (sp, "decimal", "0");
//     spHex = new QListViewItem (sp, "hexadecimal", "0h");

     dptr = new QListViewItem (lv, "Data Pointer", "");
     dptrDec = new QListViewItem (dptr, "decimal", "0");
//     dptrHex = new QListViewItem (dptr, "hexadecimal", "0h");

     pcon = new QListViewItem (lv, "Power Control", "");
     pconDec = new QListViewItem (pcon, "decimal", "0");
     pconHex = new QListViewItem (pcon, "hexadecimal", "0h");

     ints = new QListViewItem (lv, "Interrupts", "");
     
     ip = new QListViewItem (ints, "Interrupt Priority", "");
     ipDec = new QListViewItem (ip, "decimal", "0");
     ipHex = new QListViewItem (ip, "hexadecimal", "0h");
//     ipReadable = new QListViewItem (ip, "bitwise meaning", "0h");

     ie = new QListViewItem (ints, "Interrupt enable", "");
     ieDec = new QListViewItem (ie, "decimal", "0");
     ieHex = new QListViewItem (ie, "hexadecimal", "0h");
//     ieReadable = new QListViewItem (ie, "bitwise meaning", "0h");

     timers = new QListViewItem (lv, "Timer / counters", "");
     
     tmod = new QListViewItem (timers, "Timer mode", "");
     tmodDec = new QListViewItem (tmod, "decimal", "0");
     tmodHex = new QListViewItem (tmod, "hexadecimal", "0h");
//     tmodReadable = new QListViewItem (tmod, "bitwise meaning", "0h");

     tcon = new QListViewItem (timers, "Timer control", "");
     tconDec = new QListViewItem (tcon, "decimal", "0");
     tconHex = new QListViewItem (tcon, "hexadecimal", "0h");
//     tconReadable = new QListViewItem (tcon, "bitwise meaning", "0h");

     t0 = new QListViewItem (timers, "Timer 0", "");
     t0Dec = new QListViewItem (t0, "decimal", "0");
//     t0Hex = new QListViewItem (t0, "hexadecimal", "0h");

     t1 = new QListViewItem (timers, "Timer 1", "");
     t1Dec = new QListViewItem (t1, "decimal", "0");
//     t1Hex = new QListViewItem (t1, "hexadecimal", "0h");

     ser = new QListViewItem (lv, "Serial comm.", "");
     
     scon = new QListViewItem (ser, "Serial Control", "");
     sconDec = new QListViewItem (scon, "decimal", "0");
//     sconHex = new QListViewItem (scon, "hexadecimal", "0h");

     sbuf = new QListViewItem (ser, "Serial Buffer", "");
     sbufDec = new QListViewItem (sbuf, "decimal", "0");
//     sbufHex = new QListViewItem (sbuf, "hexadecimal", "0h");

     regs = new QListViewItem (lv, "R0 - R7", "");
     
     r0 = new QListViewItem (regs, "R0", "");
     r0Dec = new QListViewItem (r0, "decimal", "0");
//     r0Hex = new QListViewItem (r0, "hexadecimal", "0h");

     r1 = new QListViewItem (regs, "R1", "");
     r1Dec = new QListViewItem (r1, "decimal", "0");
//     r1Hex = new QListViewItem (r1, "hexadecimal", "0h");

     r2 = new QListViewItem (regs, "R2", "");
     r2Dec = new QListViewItem (r2, "decimal", "0");
//     r2Hex = new QListViewItem (r2, "hexadecimal", "0h");

     r3 = new QListViewItem (regs, "R3", "");
     r3Dec = new QListViewItem (r3, "decimal", "0");
//     r3Hex = new QListViewItem (r3, "hexadecimal", "0h");

     r4 = new QListViewItem (regs, "R4", "");
     r4Dec = new QListViewItem (r4, "decimal", "0");
//     r4Hex = new QListViewItem (r4, "hexadecimal", "0h");

     r5 = new QListViewItem (regs, "R5", "");
     r5Dec = new QListViewItem (r5, "decimal", "0");
//     r5Hex = new QListViewItem (r5, "hexadecimal", "0h");

     r6 = new QListViewItem (regs, "R6", "");
     r6Dec = new QListViewItem (r6, "decimal", "0");
//     r6Hex = new QListViewItem (r6, "hexadecimal", "0h");

     r7 = new QListViewItem (regs, "R7", "");
     r7Dec = new QListViewItem (r7, "decimal", "0");
//     r7Hex = new QListViewItem (r7, "hexadecimal", "0h");


     update();
}

void MemView::update ()
{    
     char buf[30];
     unsigned char tmp;
     snprintf (buf, 29, "%d", da (A));
     accDec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (A)); 
     acc->setText (1, buf);

     snprintf (buf, 29, "%d", da (B));
     bDec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (B)); 
     b->setText (1, buf);

     snprintf (buf, 29, "%d", da (PSW));
     pswDec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (PSW)); 
     pswHex->setText (1, buf);
     psw->setText (1, psw_readable (da (PSW)));
     
     snprintf (buf, 29, "%d", da (SP));
     spDec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (SP)); 
     sp->setText (1, buf);

     snprintf (buf, 29, "%d", get_dptr());
     dptrDec->setText (1, buf);
     snprintf (buf, 29, "%XH", get_dptr()); 
     dptr->setText (1, buf);

     tmp = da (PCON);
     snprintf (buf, 29, "%s%s%s%s%s",
               (tmp & 0x80 ? "SMOD " : ""),
               (tmp & 0x08 ? "GF1 " : ""),
               (tmp & 0x04 ? "GF2 " : ""),
               (tmp & 0x02 ? "PD " : ""),
               (tmp & 0x01 ? "IDL " : ""));
     pcon->setText (1, buf);
     snprintf (buf, 29, "%d", tmp);
     pconDec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)tmp); 
     pconHex->setText (1, buf);

     snprintf (buf, 29, "%d", da (IP));
     ipDec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (IP)); 
     ipHex->setText (1, buf);
     tmp = da(IP);
     snprintf (buf, 29, "%s%s%s%s%s",
               (tmp & 0x10 ? "PS " : ""),
               (tmp & 0x08 ? "PT1 " : ""),
               (tmp & 0x04 ? "PX1 " : ""),
               (tmp & 0x02 ? "PT0 " : ""),
               (tmp & 0x01 ? "PX0 " : ""));
     ip->setText (1, buf);

     snprintf (buf, 29, "%d", da (IE));
     ieDec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (IE)); 
     ieHex->setText (1, buf);
     tmp = da (IE);
     snprintf (buf, 29, "%s %s %s %s %s %s",
               (tmp & 0x80 ? "EA" : ""),
               (tmp & 0x10 ? "ES" : ""),
               (tmp & 0x08 ? "ET1" : ""),
               (tmp & 0x04 ? "EX1" : ""),
               (tmp & 0x02 ? "ET0" : ""),
               (tmp & 0x01 ? "EX0" : ""));
     ie->setText (1, buf);
     
     snprintf (buf, 29, "%d", da (TMOD));
     tmodDec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (TMOD)); 
     tmodHex->setText (1, buf);
     tmp = da (TMOD);
     snprintf (buf, 29, "T1[%s %s M%d] T0[%s %s M%d]",
               (tmp & 0x80 ? "G" : ""),
               (tmp & 0x40 ? "C" : "T"), (tmp & 0x30) >> 4,
               (tmp & 0x08 ? "G" : ""),
               (tmp & 0x04 ? "C" : "T"), (tmp & 0x03));
     tmod->setText (1, buf);

     snprintf (buf, 29, "%d", da (TCON));
     tconDec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (TCON)); 
     tconHex->setText (1, buf);
     tmp = da (TCON);
     snprintf (buf, 29, "%s%s%s%s%s%s%s%s",
               (tmp & 0x80 ? "TF1 " : ""),
               (tmp & 0x40 ? "TR1 " : ""),
               (tmp & 0x20 ? "TF0 " : ""),
               (tmp & 0x10 ? "TR0 " : ""),
               (tmp & 0x08 ? "IE1 " : ""),
               (tmp & 0x04 ? "IT1 " : ""),
               (tmp & 0x02 ? "IE0 " : ""),
               (tmp & 0x01 ? "IT0 " : ""));
     tcon->setText (1, buf);

     snprintf (buf, 29, "%d %d", da (TH0), da (TL0));
     t0Dec->setText (1, buf);
     snprintf (buf, 29, "%X%XH ", (unsigned char)da (TH0),
               (unsigned char)da (TL0)); 
     t0->setText (1, buf);

     snprintf (buf, 29, "%d %d", da (TH1), da (TL1));
     t1Dec->setText (1, buf);
     snprintf (buf, 29, "%X%XH ", (unsigned char)da (TH1),
               (unsigned char)da (TL1)); 
     t1->setText (1, buf);

     snprintf (buf, 29, "%d", da (SCON));
     sconDec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (SCON)); 
     scon->setText (1, buf);

     snprintf (buf, 29, "%d", da (sfr_info["R0"].address));
     r0Dec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (sfr_info["R0"].address)); 
     r0->setText (1, buf);

     snprintf (buf, 29, "%d", da (sfr_info["R1"].address));
     r1Dec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (sfr_info["R1"].address)); 
     r1->setText (1, buf);

     snprintf (buf, 29, "%d", da (sfr_info["R2"].address));
     r2Dec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (sfr_info["R2"].address)); 
     r2->setText (1, buf);

     snprintf (buf, 29, "%d", da (sfr_info["R3"].address));
     r3Dec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (sfr_info["R3"].address)); 
     r3->setText (1, buf);

     snprintf (buf, 29, "%d", da (sfr_info["R4"].address));
     r4Dec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (sfr_info["R4"].address)); 
     r4->setText (1, buf);

     snprintf (buf, 29, "%d", da (sfr_info["R5"].address));
     r5Dec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (sfr_info["R5"].address)); 
     r5->setText (1, buf);

     snprintf (buf, 29, "%d", da (sfr_info["R6"].address));
     r6Dec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (sfr_info["R6"].address)); 
     r6->setText (1, buf);

     snprintf (buf, 29, "%d", da (sfr_info["R7"].address));
     r7Dec->setText (1, buf);
     snprintf (buf, 29, "%XH", (unsigned char)da (sfr_info["R7"].address)); 
     r7->setText (1, buf);

}

void MemView::singleClick (QListViewItem *item, const QPoint &p, int col)
{
     if (col == 1) {
          if (item == acc) {
               MemEdit mm (A, "Accumulator", MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == b) {
               MemEdit mm (B, "B register", MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == psw) {
               MemEdit mm (PSW, "Program Status Word", MemEdit::direct, this);
               if (mm.exec ()) {
                    create_r_registers ();
                    update ();
               }
          }
          else if (item == sp) {
               MemEdit mm (SP, "SP register", MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == pcon) {
               MemEdit mm (PCON, "PCON register", MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == scon) {
               MemEdit mm (SCON, "SCON register", MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == sbuf) {
               MemEdit mm (SBUF, "SBUF register", MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == tcon) {
               MemEdit mm (TCON, "TCON register", MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == tmod) {
               MemEdit mm (TMOD, "TMOD register", MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == ip) {
               MemEdit mm (IP, "IP register", MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == ie) {
               MemEdit mm (IE, "IE register", MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == r0) {
               MemEdit mm (sfr_info["R0"].address, "R0 register",
                           MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == r1) {
               MemEdit mm (sfr_info["R1"].address, "R1 register",
                           MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == r2) {
               MemEdit mm (sfr_info["R2"].address, "R2 register",
                           MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == r3) {
               MemEdit mm (sfr_info["R3"].address, "R3 register",
                           MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == r4) {
               MemEdit mm (sfr_info["R4"].address, "R4 register",
                           MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == r5) {
               MemEdit mm (sfr_info["R5"].address, "R5 register",
                           MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == r6) {
               MemEdit mm (sfr_info["R6"].address, "R6 register",
                           MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
          else if (item == r7) {
               MemEdit mm (sfr_info["R7"].address, "R7 register",
                           MemEdit::direct, this);
               if (mm.exec ())
                    update ();
          }
     }
}
